% compute averages of the image and the global power spectrum of the
% images in the set as well as slopes.
% 
% Reference: Torralba, A., Oliva, A. (2003). Statistics of natural images
% categories. Network: computation in neural systems, 14, 391-412.

clear all
close all

% Put here the name of the folder that contains the set of images 

folder = 'pebbles/';

files = [dir([folder '*.jpg']) ;dir([folder '*.tif']) ;dir([folder '*.tiff']) ;dir([folder '*.bmp'])]; 
Nimages = length(files);

% Window to remove boundary artifatcs
g=hammingfn(256)*hammingfn(256)'; % Here I assume that images are 256x256


averageImage = 0;
averageFFT = 0;
figure

% Loop: read images and compute averages in spatial domain
for i = 1:Nimages
    % real image
    img = imread([folder files(i).name]);
    [m,n,c] = size(img); if c == 1; img = repmat(img,[1 1 3]); end
    % img = imresize(img, [256 256]); % if your images have different sizes
    % you will have to normalize them here... I assume that your images are
    % square and 256x256
    imagesc(img); drawnow
    
    % Compute FFT fast fourier transform and power spectrum
    img = double(img);
    imgBW = mean(img, 3); % put image in gray levels
    imgBW = imgBW-mean(imgBW(:)); % remove mean
    fftImg=abs(fft2(imgBW.*g)).^2; % compute power spectrum
    fftImg=fftImg/sum(fftImg(:)); % normalize to unit power
    
    averageImage = averageImage+img; % image average
    averageFFT=averageFFT+fftImg;    % power spectrum average
end
averageImage = averageImage/Nimages;
averageFFT = averageFFT/Nimages;
averageFFT = fftshift(averageFFT); % put frequency (0,0) in the center

% PLOTS:
figure
averageImage = averageImage-min(averageImage(:)); averageImage = averageImage/max(averageImage(:));
imagesc(averageImage); title('image average'); axis('square')

figure
imagesc(log(averageFFT)); colormap(gray(256)); axis('tight'); axis('square')
xlabel('f_x'); ylabel('f_y'); zlabel('log'); %view(0,90)
title('power spectrum average')


figure
surf(log(averageFFT)); shading interp; colormap(gray(256)); axis('tight'); axis('square')
xlabel('f_x'); ylabel('f_y'); zlabel('log'); %view(0,90)
title('power spectrum average')

% Plot sections:
[thresholds, ndx] = sort(-averageFFT(:)); thresholds= -thresholds;
energySections = cumsum(thresholds);
[foo, ndx90] = min(abs(energySections - .9)); th90 = thresholds(ndx90);
[foo, ndx80] = min(abs(energySections - .8)); th80 = thresholds(ndx80);
[foo, ndx60] = min(abs(energySections - .6)); th60 = thresholds(ndx60);

figure
[c,h] = contour(-128:127, -128:127, averageFFT,[th90 th80 th60]); % Here I assume that images are 256x256
xlabel('f_x (cycles per image)');ylabel('f_y (cycles per image)'); 
title('Sections that account for 60, 80 and 90 % of the image energy')
grid on
axis('square'); 
drawnow

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Slopes:
% Parameters:
fb=[0.01 0.45]; % interval of frequencies (in cycles per pixel => max value = .5)
Nangles = 4;    % Number of angles

th=linspace(0, pi, Nangles+1); th = th(1:end-1); 
n=256;  % Here I assume that images are 256x256

[fx,fy] = meshgrid(0:n-1,0:n-1);fx=fx-n/2;fy=fy-n/2; 

f=logspace(log10(fb(1)), log10(fb(2)), 20); % select 20 points along a radial axis
lf=log10(f);
theta=pi/2-th;

theta = th;
figure
subplot(121)
col='rgbcmkyrgbcmkyrgbcmkyrgbcmky';
clear h slope prefactor
% Loop on angles:
for kk=1:length(theta)
    % extract a radial section of the power spectrum
    t=theta(kk); fxi=-f*sin(t)*n; fyi=f*cos(t)*n;
    p=interp2(fx,fy,averageFFT,fxi,fyi,'linear'); 
    
    % Fit the model spectrum = prefactor * f^slope
    lp=log10(p);
    pol=polyfit(lf,lp,1); % FITTING WITH A LINE
    slope(kk)=pol(1); % this is the slope
    prefactor(kk)=exp(pol(2)); % this is the prefactor 
    
    % Plot
    plot(lf, lp, col(kk), 'linewidth', 3)
    axis('tight')
    hold on
    h(kk) = plot(lf, lf*pol(1)+pol(2), [col(kk) '-o']);
    grid on
end
xlabel('frequency (cycles per pixel in log units)')
ylabel('power spectrum (radial section)')
legend(h, num2str(theta'/pi*180)); 
title('The angles read as: f_y = 0, f_x = 90')

subplot(122)
plot(theta/pi*180, slope, 's-')
text (theta/pi*180, slope, num2str(-slope'))
xlabel('Angle (The angles read as: f_y = 0, f_x = 90)')
ylabel('slope')
title('spectrum = prefactor * f^{ slope}')


